#include <stdlib.h>
#include <math.h>
#include <GL/glew.h>
#include <FL/Fl.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Gl_Window.H>
#include "gl_draw_shader.H"

class GLTextCanvas : public Fl_Gl_Window
{
private:
	Fl_Gl_Text_Static *_canvas;
	int _canvasX, _canvasY, _mouseX, _mouseY;
	float _angle;
public:
	GLTextCanvas(int w, int h, const char *str, Fl_Align align, GLint textureNumber = -1) : Fl_Gl_Window(0, 0, w, h)
	{
		mode(FL_RGB | FL_DOUBLE | FL_ALPHA | FL_OPENGL3);
		_canvasX = 0;
		_canvasY = 0;
		_canvas = new Fl_Gl_Text_Static(this, str, _canvasX, _canvasY, w, 0, align, 0, textureNumber);
	}

	virtual ~GLTextCanvas() {delete _canvas;}

	virtual void draw()
	{
		if (!valid())
		{
            glViewport(0, 0, pixel_w(), pixel_h());
			glClearColor(0.5, 0.75, 1.0, 1.0);
			valid(1);
        }
		glClear(GL_COLOR_BUFFER_BIT);
		// We could use gl_draw_shader(...); here instead
		_canvas->draw();
	}

	virtual int handle(int event)
	{
		static int firstTime = 1;
		if (firstTime && event == FL_SHOW && shown())
		{
			firstTime = 0;
			make_current();
#ifdef __APPLE__
			glewExperimental = GL_TRUE;
#endif
			GLenum err = glewInit();
			if (err) Fl::warning("glewInit() failed returning %u", err);
		}
		switch (event)
		{
		case FL_PUSH:
			_mouseX = Fl::event_x();
			_mouseY = Fl::event_y();
			return 1;
		case FL_DRAG:
		{
			int dx = Fl::event_x() - _mouseX;
			int dy = Fl::event_y() - _mouseY;
			_mouseX = Fl::event_x();
			_mouseY = Fl::event_y();
			_canvasX += dx;
			_canvasY += dy;
			_canvas->position(_canvasX, _canvasY);
			redraw();
			return 1;
		}
		case FL_RELEASE:
			_mouseX = -1;
			_mouseY = -1;
			return 1;
		case FL_MOUSEWHEEL:
			_angle += 0.1 * Fl::event_dy();
			GLfloat matrix[] =
			{
				cos(_angle), sin(_angle), 0, 0,
				-sin(_angle), cos(_angle), 0, 0,
				0, 0, 1, 0,
				0, 0, 0, 1
			};
			_canvas->transform(matrix);
			redraw();
			return 1;
		}
		return Fl_Gl_Window::handle(event);
	}
};

int main(int argc, char *argv[])
{
	char text[] = "Drag the text to move it, use mouse wheel to rotate it";

	Fl_Window *test = new Fl_Window(400, 300, "FLTK text rendering");
	Fl_Box *box = new Fl_Box(0, 0, 400, 300, text);
	box->box(FL_FLAT_BOX);
	box->color(fl_rgb_color(127, 191, 255));
	box->align(FL_ALIGN_RIGHT | FL_ALIGN_WRAP | FL_ALIGN_INSIDE);
	box->labelfont(FL_TIMES);
	box->labelsize(48);
	test->end();
	test->resizable(box);
	test->show(argc, argv);

	Fl::use_high_res_GL(1);
	fl_font(FL_TIMES, 48);
	fl_color(FL_BLACK);
	Fl_Window *window = new Fl_Window(400, 300, "Texture text rendering");
	GLTextCanvas *canvas = new GLTextCanvas(400, 300, text, FL_ALIGN_RIGHT | FL_ALIGN_WRAP, 0);
	canvas->end();
	window->end();
	window->resizable(canvas);
	window->show(argc, argv);

	return Fl::run();
}
