#ifndef _FL_DRAW_SHADER_H
#define _FL_DRAW_SHADER_H

#include <map>
#include <string>
#include <GL/glew.h>
#include <FL/Fl.H>
#include <FL/x.H>
#include <FL/fl_draw.H>
#include <FL/Fl_Gl_Window.H>

using std::map;
using std::string;

extern const char *VERTEX_SHADER_SRC;
extern const char *FRAGMENT_SHADER_SRC;

class Fl_Gl_Text_Static
{
private:
	bool _init;
	unsigned char *_buffer;
	GLenum _dataMode;
	GLuint _texture, _vertexBuffer, _vertexArray, _program;
	GLint _projection, _transform, _image, _color, _textureNumber;
	int _x, _y, _w, _h, _parentWidth, _parentHeight;
	Fl_Color _fl_color;
	Fl_Gl_Window *_gl;
	void init();
	unsigned char *drawToTexture(const char *str, int &w, int &h, Fl_Align align, int draw_symbols);
	GLuint createTexture(GLint textureNumber, int w, int h, unsigned char *buffer);
	GLuint createVertexBuffer();
	GLuint createVertexArray(GLuint vertexBuffer);
	GLuint createShaderProgram();
	void initShaderParameters(GLuint program, GLint &projection, GLint &transform, GLint &image, GLint &color);
	void projection(int x, int y, int parentWidth, int parentHeight);
public:
	Fl_Gl_Text_Static(Fl_Gl_Window *gl, const char *str, int x, int y, int w, int h, Fl_Align align, int draw_symbols, GLint textureNumber = -1);
	virtual ~Fl_Gl_Text_Static();
	void position(int x, int y);
	void color(Fl_Color color);
	void color(uchar red, uchar green, uchar blue);
	void transform(GLfloat *matrix);
	void draw();
	friend void gl_draw_shader(const char *str, int x, int y, int w, int h, Fl_Align align, int draw_symbols);
};

extern void gl_draw_shader(const char *str, int x, int y, int w, int h, Fl_Align align, int draw_symbols);

#endif
